////////////////////////////////////////////////////////////////////////////////
//
//		Licensed Materials - Property of IBM
//  		xxxx-xxx
//		(C) Copyright IBM Corp. 2004. All rights reserved.
//		US Government Users Restricted Rights - Use, duplication or disclosure
//		restricted by GSA ADP Schedule Contract with IBM Corp.
//
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has been
// deposited with the U.S. Copyright Office.
//
////////////////////////////////////////////////////////////////////////////////
package com.ibm.debug.idebug.engine;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * The EngineProcess class describes attributes of a running debugger engine.
 * Also a EngineProcess instance will automatically add itself to the engines 
 * list in PiclEngineLauncher on creation and remove itself on destruction.
 */
public class EngineProcess {
	protected Process	_process				= null;
	protected Thread	_errorStreamThread		= null;
	protected Thread	_outputStreamThread		= null;
	protected Thread	_engineMonitorThread	= null;

	/**
	 * @param process
	 *            The process of the debugger engine
	 * @throws IllegalArgumentException
	 */
	protected EngineProcess(Process process)
			throws IllegalArgumentException {
		if (process == null)
			throw new IllegalArgumentException();

		this._process = process;

		// Start up threads to keep the engine error and output stream flushed.
		// We have to do this because some native platforms only provide a 
		// limited buffer size for standard input and output streams, failure 
		// to promptly read the output stream of the sub-process may cause the 
		// sub-process to block, and even deadlock. 
		_errorStreamThread = new Thread(new InputStreamReader(_process
				.getErrorStream()));
		_errorStreamThread.start();
		_outputStreamThread = new Thread(new InputStreamReader(_process
				.getInputStream()));
		_outputStreamThread.start();

		// This thread will wait for the engine process to end and then it
		// will remove this descriptor from the _engines list
		_engineMonitorThread = new Thread(new EngineProcessMonitor(this));
		_engineMonitorThread.start();
	}

	/**
	 * @return The process under which the debugger engine is running.
	 */
	public Process getProcess() {
		return _process;
	}

	protected class EngineProcessMonitor implements Runnable {
		EngineProcess	_engine	= null;

		public EngineProcessMonitor(EngineProcess e) {
			this._engine = e;
		}

		public void run() {
			try {
				if (!PiclEngineLauncher._engines.contains(_engine))
					PiclEngineLauncher._engines.add(_engine);

				if (_engine._process != null)
					_engine._process.waitFor();

				if (PiclEngineLauncher._engines.contains(_engine))
					PiclEngineLauncher._engines.remove(_engine);
			} catch (InterruptedException e) {
			}
		}
	}

	protected class InputStreamReader implements Runnable {
		BufferedInputStream	stream	= null;

		public InputStreamReader(InputStream in) {
			stream = new BufferedInputStream(in);
		}

		public void run() {
			int c = 0;

			try {
				while ((c = stream.read()) != -1);
			} catch (IOException e) {
			}
		}
	}

	private static final String	IBMCopyRight	= "(C) Copyright IBM Corp. 2004. All rights reserved.";
}
